global TDATA UDATA PARAMS

PARAMS.M=100;
PARAMS.c=1;
PARAMS.k=1;
PARAMS.T=10;
PARAMS.u0=10;

PARAMS.errtol=1e-8;
PARAMS.h=1e-4;
gradtol=1e-4;

x0=[1.1;1.05];

% Exact solution (data):
TDATA=linspace(0,PARAMS.T,PARAMS.M)';
r=(sqrt(PARAMS.c^2-4*PARAMS.k)-PARAMS.c)/2;
A=PARAMS.u0; B=-A*real(r)/imag(r);
UDATA=exp(real(r)*TDATA).*(A*cos(imag(r)*TDATA)+B*sin(imag(r)*TDATA));

% Check exact vs. numerical solution to ODE:
%figure;plot(TDATA, UDATA,'-o')
%options=odeset('AbsTol',PARAMS.errtol,'RelTol',PARAMS.errtol);
%figure;ode15s(@harmosc,[0 PARAMS.T], [PARAMS.u0 0],options...
%             ,PARAMS.c,PARAMS.k);

% Check "zero residual":
res0=pidobj([PARAMS.c,PARAMS.k])

% initialize figure handle
figure;itf=gcf;

if (1)
% Call Damped Gauss-Newton with Armijo rule:
disp('Damped Gauss-Newton with Armijo rule:')
tic
[x,histout,costdata] = gaussn(x0,@pidobj,gradtol);
tDGNA=toc;
x

for i=1:size(histout,1)
  disp(sprintf('norm(grad)=%d, f=%d, pullbacks=%d, its=%d',histout(i,1:4)));
end
disp(sprintf('fcns=%d, grads=%d, hess=%d, time=%g',costdata,tDGNA));

% Plot results:
figure
semilogy(histout(:,1),'-o')
xlabel('Iterations')
ylabel('Gradient Norm')
title('Damped Gauss-Newton with Armijo rule')

figure
semilogy(histout(:,2),'-o')
xlabel('Iterations')
ylabel('Function Value')
title('Damped Gauss-Newton with Armijo rule')

figure(itf)
plot(histout(1,5),histout(1,6),'ob')
hold on
h1=plot(histout(:,5),histout(:,6),'-b+');
plot(histout(end,5),histout(end,6),'xb')
xlabel('c')
ylabel('k')
title('Iteration history')

end

if (1)
% Call Steepest Descent with Armijo rule:
disp('Steepest Descent with Armijo rule:')
tic
[x,histout,costdata] = steep(x0,@pidobj,gradtol);
tSDA=toc;
x

for i=1:size(histout,1)
  disp(sprintf('norm(grad)=%d, f=%d, pullbacks=%d, its=%d',histout(i,1:4)));
end
disp(sprintf('fcns=%d, grads=%d, hess=%d, time=%g',costdata,tSDA));

% Plot results:
figure
semilogy(histout(:,1),'-o')
xlabel('Iterations')
ylabel('Gradient Norm')
title('Steepest Descent with Armijo rule')

figure
semilogy(histout(:,2),'-o')
xlabel('Iterations')
ylabel('Function Value')
title('Steepest Descent with Armijo rule')


figure(itf)
plot(histout(1,5),histout(1,6),'go')
hold on
h2=plot(histout(:,5),histout(:,6),'--g+');
plot(histout(end,5),histout(end,6),'gx')

xlabel('c')
ylabel('k')
title('Iteration history')

end

if (1)
% Call Newton Trust Region:
disp('Newton Trust Region')
tic
[x,histout,costdata] = ntrust(x0,@pidobj,gradtol);
tNTR=toc;
x

for i=1:size(histout,1)
  disp(sprintf('norm(grad)=%d, f=%d, pullbacks=%d, its=%d',histout(i,1:4)));
end
disp(sprintf('fcns=%d, grads=%d, hess=%d, time=%g',costdata,tNTR));

% Plot results:
figure
semilogy(histout(:,1),'-o')
xlabel('Iterations')
ylabel('Gradient Norm')
title('Newton Trust Region')

figure
semilogy(histout(:,2),'-o')
xlabel('Iterations')
ylabel('Function Value')
title('Newton Trust Region')
end

figure(itf)
plot(histout(1,5),histout(1,6),'ro')
hold on
h3=plot(histout(:,5),histout(:,6),'-.r+');
plot(histout(end,5),histout(end,6),'rx')

xlabel('c')
ylabel('k')
title('Iteration history')
legend([h1,h2,h3],'Gauss-Newton','Steepest Descent','Newton Trust Region')

% plot contour
k=[.95:.01:1.1];
c=[.95:.01:1.15];
A=(0*k')*(0*c);
for i=1:length(c),
  for j=1:length(k),
    A(j,i)=pidobj([c(i),k(j)]);
  end
end
%figure; surf(c,k,A)
[C,K]=meshgrid(c,k);
figure(itf); contour(C,K,A,30)
axis('tight')
